/*
 * Copyright (C) 1999 Toru Hoshina
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __DC210_H__
#define __DC210_H__

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define CAMERAEPOC 852094800L

struct camera_stat {
   unsigned char dummy;
   unsigned char cameraTypeId;
   unsigned char firmwareMajor;
   unsigned char firmwareMinor;
   unsigned char blah1;
   unsigned char blah2;
   unsigned char blah3;
   unsigned char blah4;
   unsigned char batteryStatusId;
   unsigned char acStatusId;
   short reserve1;
   long cameraTime;
   unsigned char zoomMode;
   unsigned char reserve2;
   unsigned char flashCharged;
   unsigned char compressionModeId;
   unsigned char flashMode;
   unsigned char exposureCompensation;
   unsigned char pictureSize;
   unsigned char fileType;
   unsigned char reserve3;
   short totalPicturesTaken;
   short totalFlashesFired;
   unsigned char padding1[27];
   unsigned char numPictures;
   unsigned char padding2[32];
   unsigned char cameraIdent[60];
};

struct picture_info {
   unsigned char dummy[3];
   unsigned char resolution;
   unsigned char compression;
   unsigned char dummy2[2];
   unsigned char pictureNumber;
   long fileSize;
   long elapsedTime;
   unsigned char padding[16];
   unsigned char fileName[13];
};

struct pinfo_lst {
   struct pinfo_lst *prev;
   struct pinfo_lst *next;
   unsigned char resolution[12];
   unsigned char compression[12];
   unsigned char number[12];
   unsigned char filesize[12];
   unsigned char filename[80];
};

extern struct camera_stat status;

int init_dc210();
void destroy_dc210();
int load_status();
unsigned char *get_resolution(int id);
unsigned char *get_flashmode(int id);
unsigned char *get_comprmode(int id);
unsigned char *get_cameratype(int id);
unsigned char *get_batterystat(int id);
unsigned char *get_acstat(int id);
unsigned char *get_cameratime(unsigned char *s);
int get_picture_info(int no, struct pinfo_lst *plst);
int get_picture(int no, struct pinfo_lst *plst, unsigned char *fname);
int get_thumbnail(int no, struct pinfo_lst *plst);
int del_picture(int no);
int del_all_picture();
int set_camera_id(unsigned char *cameraID);
int take_picture();

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __DC210_H__ */
