/*
 * Copyright (C) 1999 Toru Hoshina
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <gtk/gtk.h>

#include "main.h"
#include "dialog.h"
#include "dc210.h"

GtkWidget *entry;
GtkWidget *spinner;

gint entry_dialog(const char *labelString,
 		  char *button1, 
 		  char *button2, 
 		  char *string
 		  )
{
   GtkWidget *dialogWindow;
   GtkWidget *label;
   GtkWidget *button;
   struct entrydlg value;
   char* buttonlabel[2];
   buttonlabel[0] = button1;
   buttonlabel[1] = button2;

   dialogWindow = gtk_dialog_new();
   gtk_container_border_width (GTK_CONTAINER(GTK_DIALOG(dialogWindow)->vbox),
			       10);

   gtk_window_set_title (GTK_WINDOW (dialogWindow), "gkdcpi");

   label = gtk_label_new (labelString);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialogWindow)->vbox), label, TRUE,
 		      TRUE, 0);
   gtk_widget_show (label);

   entry = gtk_entry_new();
   gtk_entry_set_text (GTK_ENTRY (entry), "");
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialogWindow)->vbox), entry, TRUE,
 		      TRUE, 0);
   gtk_widget_show (entry);

   button = gtk_button_new_with_label(button1!=NULL?button1:"OK");
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
                       GTK_SIGNAL_FUNC (entry_click), (gpointer)&value);
   gtk_object_set_user_data( GTK_OBJECT(button), (gpointer)(-1) );
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			      GTK_SIGNAL_FUNC (gtk_widget_destroy),
			      GTK_OBJECT(dialogWindow));
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialogWindow)->action_area),
		       button, TRUE, TRUE, 0);
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
   gtk_widget_grab_default (button);
   gtk_widget_show (button);

   button = gtk_button_new_with_label(button2!=NULL?button2:"Cancel");
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
                       GTK_SIGNAL_FUNC (entry_click), (gpointer)&value);
   gtk_object_set_user_data( GTK_OBJECT(button), (gpointer)(0) );
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			      GTK_SIGNAL_FUNC (gtk_widget_destroy),
			      GTK_OBJECT(dialogWindow));
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialogWindow)->action_area),
		       button, TRUE, TRUE, 0);
   gtk_widget_show (button);

   gtk_widget_show(dialogWindow);

#ifdef MODAL
   gtk_grab_add( dialogWindow );
#endif
   value.ret_val = 2;
   value.ret_str = string;
   while ( value.ret_val == 2 )
     gtk_main_iteration();
#ifdef MODAL
   //gtk_grab_remove( dialogWindow ); //?? Doesn't need ??
#endif
   return value.ret_val; 
}

void entry_click (GtkWidget *widget, gpointer value)
{
   gint id = (gint)gtk_object_get_user_data( GTK_OBJECT(widget) );
   //g_print("button%d is clicked\n", id+2);
   ((struct entrydlg *)(value))->ret_val = id;
   strcpy(((struct entrydlg *)(value))->ret_str, gtk_entry_get_text(GTK_ENTRY(entry)));
}

gint alert_dialog(const char *labelString,
 		  char *button1, 
 		  char *button2, 
 		  char *button3
 		  )
{
   char *defaultbutton = "Yes";
   GtkWidget *dialogWindow;
   GtkWidget *label;
   GtkWidget *button;
   gint value;
   int i; 
   char* buttonlabel[3];
   buttonlabel[0] = button1;
   buttonlabel[1] = button2;
   buttonlabel[2] = button3;

   dialogWindow = gtk_dialog_new();
   gtk_container_border_width (GTK_CONTAINER(GTK_DIALOG(dialogWindow)->vbox),
			       10);

   gtk_window_set_title (GTK_WINDOW (dialogWindow), "gkdcpi");

   label = gtk_label_new (labelString);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialogWindow)->vbox), label, TRUE,
 		      TRUE, 0);
   gtk_widget_show (label);

   if(button1 == NULL && button2 == NULL && button3 == NULL ) {
      button3 = defaultbutton;
   }
   for ( i = 0; i < 3; i++ ) {
      if(buttonlabel[i] != NULL) {
	 button = gtk_button_new_with_label(buttonlabel[i]);
	 gtk_signal_connect (GTK_OBJECT (button), "clicked",
			     GTK_SIGNAL_FUNC (dialog_click), (gpointer)&value);
	 gtk_object_set_user_data( GTK_OBJECT(button), (gpointer)(-1+i) );
	 gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
				    GTK_SIGNAL_FUNC (gtk_widget_destroy),
				    GTK_OBJECT(dialogWindow));
	 gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialogWindow)->action_area),
			     button, TRUE, TRUE, 0);
	 gtk_widget_show (button);
      }
   }
   gtk_widget_show(dialogWindow);

#ifdef MODAL
   gtk_grab_add( dialogWindow );
#endif
   value = 2;
   while ( value == 2 )
     gtk_main_iteration();
#ifdef MODAL
   //gtk_grab_remove( dialogWindow ); //?? Doesn't need ??
#endif
   return value; 
}

void dialog_click (GtkWidget *widget, gpointer value)
{
   gint id = (gint)gtk_object_get_user_data( GTK_OBJECT(widget) );
   //g_print("button%d is clicked\n", id+2);
   *((gint*)value) = id;
}

void
devicebutton_toggled (GtkWidget *widget, gpointer value)
{
   device_no = (gint)value;
}

gint device_dialog()
{
   GtkWidget *dialogWindow;
   GtkWidget *ttybutton[4];
   GtkWidget *box1;
   GtkWidget *box2;
   GtkWidget *button;
   gint value;

   /* Create Dialog Window */
   dialogWindow = gtk_dialog_new();
   gtk_container_border_width (GTK_CONTAINER(GTK_DIALOG(dialogWindow)->vbox),
			       10);

   gtk_window_set_title (GTK_WINDOW (dialogWindow), "Select Device");

   box1 = gtk_vbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialogWindow)->vbox), 
		       box1, TRUE, TRUE, 0);
   gtk_widget_show (box1);

   box2 = gtk_vbox_new (FALSE, 10);
   gtk_container_border_width (GTK_CONTAINER (box2), 10);
   gtk_box_pack_start (GTK_BOX (box1), box2, TRUE, TRUE, 0);
   gtk_widget_show (box2);

   ttybutton[0] = gtk_radio_button_new_with_label (NULL, "/dev/ttyS0");
   gtk_signal_connect (GTK_OBJECT (ttybutton[0]), "toggled",
		       GTK_SIGNAL_FUNC (devicebutton_toggled),
		       (gpointer)0);
   gtk_box_pack_start (GTK_BOX (box2), ttybutton[0], TRUE, TRUE, 0);
   gtk_widget_show (ttybutton[0]);

   ttybutton[1] = gtk_radio_button_new_with_label (
		    gtk_radio_button_group (GTK_RADIO_BUTTON (ttybutton[0])),
						   "/dev/ttyS1");
   gtk_signal_connect (GTK_OBJECT (ttybutton[1]), "toggled",
		       GTK_SIGNAL_FUNC (devicebutton_toggled),
		       (gpointer)1);
   gtk_box_pack_start (GTK_BOX (box2), ttybutton[1], TRUE, TRUE, 0);
   gtk_widget_show (ttybutton[1]);

   ttybutton[2] = gtk_radio_button_new_with_label (
		    gtk_radio_button_group (GTK_RADIO_BUTTON (ttybutton[0])),
						   "/dev/ttyS2");
   gtk_signal_connect (GTK_OBJECT (ttybutton[2]), "toggled",
		       GTK_SIGNAL_FUNC (devicebutton_toggled),
		       (gpointer)2);
   gtk_box_pack_start (GTK_BOX (box2), ttybutton[2], TRUE, TRUE, 0);
   gtk_widget_show (ttybutton[2]);

   ttybutton[3] = gtk_radio_button_new_with_label (
		    gtk_radio_button_group (GTK_RADIO_BUTTON (ttybutton[0])),
						   "/dev/ttyS3");
   gtk_signal_connect (GTK_OBJECT (ttybutton[3]), "toggled",
		       GTK_SIGNAL_FUNC (devicebutton_toggled),
		       (gpointer)3);
   gtk_box_pack_start (GTK_BOX (box2), ttybutton[3], TRUE, TRUE, 0);
   gtk_widget_show (ttybutton[3]);

   gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (ttybutton[device_no]), TRUE);

   button = gtk_button_new_with_label("Yes");
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
		       GTK_SIGNAL_FUNC (dialog_click), (gpointer)&value);
   gtk_object_set_user_data( GTK_OBJECT(button), (gpointer)(-1) );
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			      GTK_SIGNAL_FUNC (gtk_widget_destroy),
			      GTK_OBJECT(dialogWindow));
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialogWindow)->action_area),
		       button, TRUE, TRUE, 0);
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
   gtk_widget_grab_default (button);
   gtk_widget_show (button);

   button = gtk_button_new_with_label("No");
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
		       GTK_SIGNAL_FUNC (dialog_click), (gpointer)&value);
   gtk_object_set_user_data( GTK_OBJECT(button), (gpointer)(0) );
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			      GTK_SIGNAL_FUNC (gtk_widget_destroy),
			      GTK_OBJECT(dialogWindow));
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialogWindow)->action_area),
		       button, TRUE, TRUE, 0);
   gtk_widget_show (button);

   gtk_widget_show(dialogWindow);

#ifdef MODAL
   gtk_grab_add( dialogWindow );
#endif
   value = 2;
   while ( value == 2 )
     gtk_main_iteration();
#ifdef MODAL
   //gtk_grab_remove( dialogWindow ); //?? Doesn't need ??
#endif
   return value; 
}

void
ratiobutton_toggled (GtkWidget *widget, gpointer value)
{
   switch((gint)value)
     {
      case 0:
	ratio_no = 9600;
	break ;
      case 1:
	ratio_no = 19200;
	break ;
      case 2:
	ratio_no = 38400;
	break ;
      case 3:
	ratio_no = 57600;
	break ;
      case 4:
	ratio_no = 115200;
	break ;
      default:
	ratio_no = 9600;
	break ;
     }
}

gint ratio_dialog()
{
   GtkWidget *dialogWindow;
   GtkWidget *ratiobutton[5];
   GtkWidget *box1;
   GtkWidget *box2;
   GtkWidget *button;
   gint value;

   /* Create Dialog Window */
   dialogWindow = gtk_dialog_new();
   gtk_container_border_width (GTK_CONTAINER(GTK_DIALOG(dialogWindow)->vbox),
			       10);

   gtk_window_set_title (GTK_WINDOW (dialogWindow), "Select Ratio");

   box1 = gtk_vbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialogWindow)->vbox), 
		       box1, TRUE, TRUE, 0);
   gtk_widget_show (box1);

   box2 = gtk_vbox_new (FALSE, 10);
   gtk_container_border_width (GTK_CONTAINER (box2), 10);
   gtk_box_pack_start (GTK_BOX (box1), box2, TRUE, TRUE, 0);
   gtk_widget_show (box2);

   ratiobutton[0] = gtk_radio_button_new_with_label (NULL, "9600");
   gtk_signal_connect (GTK_OBJECT (ratiobutton[0]), "toggled",
		       GTK_SIGNAL_FUNC (ratiobutton_toggled),
		       (gpointer)0);
   gtk_box_pack_start (GTK_BOX (box2), ratiobutton[0], TRUE, TRUE, 0);
   gtk_widget_show (ratiobutton[0]);

   ratiobutton[1] = gtk_radio_button_new_with_label (
		    gtk_radio_button_group (GTK_RADIO_BUTTON (ratiobutton[0])),
						   "19200");
   gtk_signal_connect (GTK_OBJECT (ratiobutton[1]), "toggled",
		       GTK_SIGNAL_FUNC (ratiobutton_toggled),
		       (gpointer)1);
   gtk_box_pack_start (GTK_BOX (box2), ratiobutton[1], TRUE, TRUE, 0);
   gtk_widget_show (ratiobutton[1]);

   ratiobutton[2] = gtk_radio_button_new_with_label (
		    gtk_radio_button_group (GTK_RADIO_BUTTON (ratiobutton[0])),
						   "38400");
   gtk_signal_connect (GTK_OBJECT (ratiobutton[2]), "toggled",
		       GTK_SIGNAL_FUNC (ratiobutton_toggled),
		       (gpointer)2);
   gtk_box_pack_start (GTK_BOX (box2), ratiobutton[2], TRUE, TRUE, 0);
   gtk_widget_show (ratiobutton[2]);

   ratiobutton[3] = gtk_radio_button_new_with_label (
		    gtk_radio_button_group (GTK_RADIO_BUTTON (ratiobutton[0])),
						   "57600");
   gtk_signal_connect (GTK_OBJECT (ratiobutton[3]), "toggled",
		       GTK_SIGNAL_FUNC (ratiobutton_toggled),
		       (gpointer)3);
   gtk_box_pack_start (GTK_BOX (box2), ratiobutton[3], TRUE, TRUE, 0);
   gtk_widget_show (ratiobutton[3]);

   ratiobutton[4] = gtk_radio_button_new_with_label (
		    gtk_radio_button_group (GTK_RADIO_BUTTON (ratiobutton[0])),
						   "115200");
   gtk_signal_connect (GTK_OBJECT (ratiobutton[4]), "toggled",
		       GTK_SIGNAL_FUNC (ratiobutton_toggled),
		       (gpointer)4);
   gtk_box_pack_start (GTK_BOX (box2), ratiobutton[4], TRUE, TRUE, 0);
   gtk_widget_show (ratiobutton[4]);

   switch( ratio_no )
     {
      case 9600:
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (ratiobutton[0]), TRUE);
	break ;
      case 19200:
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (ratiobutton[1]), TRUE);
	break ;
      case 38400:
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (ratiobutton[2]), TRUE);
	break ;
      case 57600:
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (ratiobutton[3]), TRUE);
	break ;
      case 115200:
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (ratiobutton[4]), TRUE);
	break ;
      default:
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (ratiobutton[0]), TRUE);
	break ;
     }

   button = gtk_button_new_with_label("Yes");
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
		       GTK_SIGNAL_FUNC (dialog_click), (gpointer)&value);
   gtk_object_set_user_data( GTK_OBJECT(button), (gpointer)(-1) );
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			      GTK_SIGNAL_FUNC (gtk_widget_destroy),
			      GTK_OBJECT(dialogWindow));
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialogWindow)->action_area),
		       button, TRUE, TRUE, 0);
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
   gtk_widget_grab_default (button);
   gtk_widget_show (button);

   button = gtk_button_new_with_label("No");
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
		       GTK_SIGNAL_FUNC (dialog_click), (gpointer)&value);
   gtk_object_set_user_data( GTK_OBJECT(button), (gpointer)(0) );
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			      GTK_SIGNAL_FUNC (gtk_widget_destroy),
			      GTK_OBJECT(dialogWindow));
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialogWindow)->action_area),
		       button, TRUE, TRUE, 0);
   gtk_widget_show (button);

   gtk_widget_show(dialogWindow);

#ifdef MODAL
   gtk_grab_add( dialogWindow );
#endif
   value = 2;
   while ( value == 2 )
     gtk_main_iteration();
#ifdef MODAL
   //gtk_grab_remove( dialogWindow ); //?? Doesn't need ??
#endif
   return value; 
}

gint status_dialog()
{
   GtkWidget *dialogWindow;
   GtkWidget *box1;
   GtkWidget *box2;
   GtkWidget *box3;
   GtkWidget *button;
   GtkWidget *label;
   gint value;
   char temp[80];

   /* Create Dialog Window */
   dialogWindow = gtk_dialog_new();
   gtk_container_border_width (GTK_CONTAINER(GTK_DIALOG(dialogWindow)->vbox),
			       10);

   gtk_window_set_title (GTK_WINDOW (dialogWindow), "Camera Status");

   box1 = gtk_vbox_new (FALSE, 0);
   gtk_container_border_width (GTK_CONTAINER (box1), 0);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialogWindow)->vbox), box1, TRUE,
 		      TRUE, 0);
   gtk_widget_show (box1);

   box2 = gtk_vbox_new (FALSE, 10);
   gtk_container_border_width (GTK_CONTAINER (box2), 10);
   gtk_box_pack_start (GTK_BOX (box1), box2, FALSE, TRUE, 0);
   gtk_widget_show (box2);

   box3 = gtk_hbox_new (FALSE, 0);
   gtk_container_border_width (GTK_CONTAINER (box3), 0);
   gtk_box_pack_start (GTK_BOX (box2), box3, FALSE, TRUE, 0);
   gtk_widget_show (box3);

   label = gtk_label_new("Camera Model: ");
   gtk_box_pack_start (GTK_BOX (box3), label, FALSE, TRUE, 0);
   gtk_widget_show (label);

   label = gtk_label_new(get_cameratype(status.cameraTypeId));
   gtk_box_pack_start (GTK_BOX (box3), label, TRUE, TRUE, 0);
   gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
   gtk_widget_show (label);

   box3 = gtk_hbox_new (FALSE, 0);
   gtk_container_border_width (GTK_CONTAINER (box3), 0);
   gtk_box_pack_start (GTK_BOX (box2), box3, FALSE, TRUE, 0);
   gtk_widget_show (box3);

   label = gtk_label_new("Camera Identification: ");
   gtk_box_pack_start (GTK_BOX (box3), label, FALSE, TRUE, 0);
   gtk_widget_show (label);

   label = gtk_label_new(status.cameraIdent);
   gtk_box_pack_start (GTK_BOX (box3), label, TRUE, TRUE, 0);
   gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
   gtk_widget_show (label);

   box3 = gtk_hbox_new (FALSE, 0);
   gtk_container_border_width (GTK_CONTAINER (box3), 0);
   gtk_box_pack_start (GTK_BOX (box2), box3, FALSE, TRUE, 0);
   gtk_widget_show (box3);

   label = gtk_label_new("Camera Time: ");
   gtk_box_pack_start (GTK_BOX (box3), label, FALSE, TRUE, 0);
   gtk_widget_show (label);

   label = gtk_label_new(get_cameratime(temp));
   gtk_box_pack_start (GTK_BOX (box3), label, TRUE, TRUE, 0);
   gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
   gtk_widget_show (label);

   box3 = gtk_hbox_new (FALSE, 0);
   gtk_container_border_width (GTK_CONTAINER (box3), 0);
   gtk_box_pack_start (GTK_BOX (box2), box3, FALSE, TRUE, 0);
   gtk_widget_show (box3);

   label = gtk_label_new("Battery Level: ");
   gtk_box_pack_start (GTK_BOX (box3), label, FALSE, TRUE, 0);
   gtk_widget_show (label);

   label = gtk_label_new(get_batterystat(status.batteryStatusId));
   gtk_box_pack_start (GTK_BOX (box3), label, TRUE, TRUE, 0);
   gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
   gtk_widget_show (label);

   box3 = gtk_hbox_new (FALSE, 0);
   gtk_container_border_width (GTK_CONTAINER (box3), 0);
   gtk_box_pack_start (GTK_BOX (box2), box3, FALSE, TRUE, 0);
   gtk_widget_show (box3);

   label = gtk_label_new("AC Power: ");
   gtk_box_pack_start (GTK_BOX (box3), label, FALSE, TRUE, 0);
   gtk_widget_show (label);

   label = gtk_label_new(get_acstat(status.acStatusId));
   gtk_box_pack_start (GTK_BOX (box3), label, TRUE, TRUE, 0);
   gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
   gtk_widget_show (label);

   box3 = gtk_hbox_new (FALSE, 0);
   gtk_container_border_width (GTK_CONTAINER (box3), 0);
   gtk_box_pack_start (GTK_BOX (box2), box3, FALSE, TRUE, 0);
   gtk_widget_show (box3);

   label = gtk_label_new("Pictures in Camera: ");
   gtk_box_pack_start (GTK_BOX (box3), label, FALSE, TRUE, 0);
   gtk_widget_show (label);

   sprintf(temp, "%d", status.numPictures);
   label = gtk_label_new(temp);
   gtk_box_pack_start (GTK_BOX (box3), label, TRUE, TRUE, 0);
   gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
   gtk_widget_show (label);

   box3 = gtk_hbox_new (FALSE, 0);
   gtk_container_border_width (GTK_CONTAINER (box3), 0);
   gtk_box_pack_start (GTK_BOX (box2), box3, FALSE, TRUE, 0);
   gtk_widget_show (box3);

   label = gtk_label_new("Flash Charged: ");
   gtk_box_pack_start (GTK_BOX (box3), label, FALSE, TRUE, 0);
   gtk_widget_show (label);

   label = gtk_label_new(status.flashCharged ? "Yes" : "No");
   gtk_box_pack_start (GTK_BOX (box3), label, TRUE, TRUE, 0);
   gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
   gtk_widget_show (label);

   box3 = gtk_hbox_new (FALSE, 0);
   gtk_container_border_width (GTK_CONTAINER (box3), 0);
   gtk_box_pack_start (GTK_BOX (box2), box3, FALSE, TRUE, 0);
   gtk_widget_show (box3);

   label = gtk_label_new("Flash Mode: ");
   gtk_box_pack_start (GTK_BOX (box3), label, FALSE, TRUE, 0);
   gtk_widget_show (label);

   label = gtk_label_new(get_flashmode(status.flashMode));
   gtk_box_pack_start (GTK_BOX (box3), label, TRUE, TRUE, 0);
   gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
   gtk_widget_show (label);

   box3 = gtk_hbox_new (FALSE, 0);
   gtk_container_border_width (GTK_CONTAINER (box3), 0);
   gtk_box_pack_start (GTK_BOX (box2), box3, FALSE, TRUE, 0);
   gtk_widget_show (box3);

   label = gtk_label_new("Compression Mode: ");
   gtk_box_pack_start (GTK_BOX (box3), label, FALSE, TRUE, 0);
   gtk_widget_show (label);

   label = gtk_label_new(get_comprmode(status.compressionModeId));
   gtk_box_pack_start (GTK_BOX (box3), label, TRUE, TRUE, 0);
   gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
   gtk_widget_show (label);

   box3 = gtk_hbox_new (FALSE, 0);
   gtk_container_border_width (GTK_CONTAINER (box3), 0);
   gtk_box_pack_start (GTK_BOX (box2), box3, FALSE, TRUE, 0);
   gtk_widget_show (box3);

   label = gtk_label_new("Total Pictures: ");
   gtk_box_pack_start (GTK_BOX (box3), label, FALSE, TRUE, 0);
   gtk_widget_show (label);

   sprintf(temp, "%d", status.totalPicturesTaken);
   label = gtk_label_new(temp);
   gtk_box_pack_start (GTK_BOX (box3), label, TRUE, TRUE, 0);
   gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
   gtk_widget_show (label);

   box3 = gtk_hbox_new (FALSE, 0);
   gtk_container_border_width (GTK_CONTAINER (box3), 0);
   gtk_box_pack_start (GTK_BOX (box2), box3, FALSE, TRUE, 0);
   gtk_widget_show (box3);

   label = gtk_label_new("Total Flashes: ");
   gtk_box_pack_start (GTK_BOX (box3), label, FALSE, TRUE, 0);
   gtk_widget_show (label);

   sprintf(temp, "%d", status.totalFlashesFired);
   label = gtk_label_new(temp);
   gtk_box_pack_start (GTK_BOX (box3), label, TRUE, TRUE, 0);
   gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
   gtk_widget_show (label);

   button = gtk_button_new_with_label("Close");
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
		       GTK_SIGNAL_FUNC (dialog_click), (gpointer)&value);
   gtk_object_set_user_data( GTK_OBJECT(button), (gpointer)(-1) );
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			      GTK_SIGNAL_FUNC (gtk_widget_destroy),
			      GTK_OBJECT(dialogWindow));
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialogWindow)->action_area),
		       button, TRUE, TRUE, 0);
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
   gtk_widget_grab_default (button);
   gtk_widget_show (button);

   gtk_widget_show(dialogWindow);

#ifdef MODAL
   gtk_grab_add( dialogWindow );
#endif
   value = 2;
   while ( value == 2 )
     gtk_main_iteration();
#ifdef MODAL
   //gtk_grab_remove( dialogWindow ); //?? Doesn't need ??
#endif
   return value; 
}

gint timer_dialog(gint *delay)
{
   GtkWidget *dialogWindow;
   GtkWidget *box;
   GtkWidget *label;
   GtkWidget *button;
   GtkAdjustment *adj;
   struct timerdlg value;

   *delay = 0;

   dialogWindow = gtk_dialog_new();
   gtk_container_border_width (GTK_CONTAINER(GTK_DIALOG(dialogWindow)->vbox),
			       10);

   gtk_window_set_title (GTK_WINDOW (dialogWindow), "Set Timer");

   box = gtk_hbox_new (FALSE, 0);
   gtk_container_border_width (GTK_CONTAINER (box), 0);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dialogWindow)->vbox), box,
		       FALSE, TRUE, 0);
   gtk_widget_show (box);

   label = gtk_label_new ("Second:");
   gtk_box_pack_start (GTK_BOX(box), label, FALSE, TRUE, 0);
   gtk_widget_show (label);

   adj = (GtkAdjustment *) gtk_adjustment_new (0.0, 0.0, 60.0, 1.0, 5.0, 0.0);
   spinner = gtk_spin_button_new (adj, 0, 0);
   gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (spinner), TRUE);
   gtk_box_pack_start (GTK_BOX(box), spinner, FALSE, TRUE, 0);
   gtk_widget_show (spinner);

   button = gtk_button_new_with_label("OK");
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
                       GTK_SIGNAL_FUNC (timer_click), (gpointer)&value);
   gtk_object_set_user_data( GTK_OBJECT(button), (gpointer)(-1) );
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			      GTK_SIGNAL_FUNC (gtk_widget_destroy),
			      GTK_OBJECT(dialogWindow));
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialogWindow)->action_area),
		       button, TRUE, TRUE, 0);
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
   gtk_widget_grab_default (button);
   gtk_widget_show (button);

   button = gtk_button_new_with_label("Cancel");
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
                       GTK_SIGNAL_FUNC (timer_click), (gpointer)&value);
   gtk_object_set_user_data( GTK_OBJECT(button), (gpointer)(0) );
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			      GTK_SIGNAL_FUNC (gtk_widget_destroy),
			      GTK_OBJECT(dialogWindow));
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialogWindow)->action_area),
		       button, TRUE, TRUE, 0);
   gtk_widget_show (button);

   gtk_widget_show(dialogWindow);

#ifdef MODAL
   gtk_grab_add( dialogWindow );
#endif
   value.ret_val = 2;
   value.delay = delay;
   while ( value.ret_val == 2 )
     gtk_main_iteration();
#ifdef MODAL
   //gtk_grab_remove( dialogWindow ); //?? Doesn't need ??
#endif
   return value.ret_val; 
 }

void timer_click (GtkWidget *widget, gpointer value)
{
   gint id = (gint)gtk_object_get_user_data( GTK_OBJECT(widget) );
   GtkSpinButton *spin = GTK_SPIN_BUTTON (spinner);
   //g_print("button%d is clicked\n", id+2);
   ((struct timerdlg *)(value))->ret_val = id;
   *(((struct timerdlg *)(value))->delay) = gtk_spin_button_get_value_as_int (spin);
}

gint thumbnail_dialog(char *fname)
{
   GtkWidget *dialogWindow;
   GtkWidget *box1;
   GtkWidget *box2;
   GtkWidget *button;
   GtkWidget *pixmapwid;
   GdkPixmap *pixmap;
   GdkBitmap *mask;
   GtkStyle *style;
   gint value;

   dialogWindow = gtk_dialog_new();
   gtk_window_position (GTK_WINDOW (dialogWindow), GTK_WIN_POS_MOUSE);

   gtk_container_border_width (GTK_CONTAINER(GTK_DIALOG(dialogWindow)->vbox),
			       10);

   gtk_window_set_title (GTK_WINDOW (dialogWindow), "Thumbnail");

   box1 = gtk_vbox_new (FALSE, 0);
   gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialogWindow)->vbox), box1);
   gtk_widget_show (box1);

   box2 = gtk_vbox_new (FALSE, 0);
   gtk_container_border_width (GTK_CONTAINER (box2), 0);
   gtk_box_pack_start (GTK_BOX (box1), box2, TRUE, TRUE, 0);
   gtk_widget_show (box2);

   style=gtk_widget_get_style(box2);

   pixmap = gdk_pixmap_create_from_xpm (window->window, &mask,
					&style->bg[GTK_STATE_NORMAL],
					fname);
   pixmapwid = gtk_pixmap_new (pixmap, mask);
   remove(fname);

   gtk_container_add (GTK_CONTAINER (box2), pixmapwid);
   gtk_widget_show (pixmapwid);

   button = gtk_button_new_with_label("Close");
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
                       GTK_SIGNAL_FUNC (dialog_click), (gpointer)&value);
   gtk_object_set_user_data( GTK_OBJECT(button), (gpointer)(-1) );
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			      GTK_SIGNAL_FUNC (gtk_widget_destroy),
			      GTK_OBJECT(dialogWindow));
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialogWindow)->action_area),
		       button, TRUE, TRUE, 0);
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
   gtk_widget_grab_default (button);
   gtk_widget_show (button);

   gtk_widget_show(dialogWindow);

#ifdef MODAL
   gtk_grab_add( dialogWindow );
#endif
   value = 2;
   while ( value == 2 )
     gtk_main_iteration();
#ifdef MODAL
   //gtk_grab_remove( dialogWindow ); //?? Doesn't need ??
#endif
   return value;
 }
