/*
 * Copyright (C) 1999 Toru Hoshina
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <string.h>

#include <gtk/gtk.h>

#include "main.h"
#include "list.h"
#include "dialog.h"
#include "menu.h"
#include "filesel.h"
#include "dc210.h"
#include "lib.h"
#include "progress.h"

void
select_list (GtkWidget *widget,
	     gint row, 
	     gint column, 
	     GdkEventButton * bevent)
{
   gint i;
   gchar *text;

   for (i = 0; i < 5; i++)
     {
	gtk_clist_get_text (GTK_CLIST (widget), row, i, &text);
     }

   list_selected_row = row;

   if (bevent)
     {
     if (bevent->button == 3)
	  {
	     gtk_menu_popup (GTK_MENU(context_menu), NULL, NULL, NULL, NULL,
			     bevent->button, bevent->time);
	     return ;
	  }
     }
}

void
init_list()
{
   static char *titles[] =
     {
	"No.",
	"Name",
	"Size",
	"Pixel",
	"Compression"
     };

   GtkWidget *box1;
   GtkWidget *box2;
#ifndef GTK_HAVE_FEATURES_1_1_0
   GtkWidget *box3;
#else
   GtkWidget *scrolled_win;
#endif
   GtkWidget *hbox;
   GtkWidget *button;
   GtkWidget *dirbox;
   GtkWidget *configbox;
   GtkWidget *label;

   box1 = gtk_vbox_new (FALSE, 0);
   gtk_container_border_width (GTK_CONTAINER (box1), 0);
   gtk_box_pack_start (GTK_BOX (main_vbox), box1, FALSE, TRUE, 0);
   gtk_widget_show (box1);

   box2 = gtk_vbox_new (FALSE, 10);
   gtk_container_border_width (GTK_CONTAINER (box2), 10);
   gtk_box_pack_start (GTK_BOX (box1), box2, FALSE, TRUE, 0);
   gtk_widget_show (box2);

   configbox = gtk_hbox_new (FALSE, 0);
   gtk_container_border_width (GTK_CONTAINER (configbox), 0);
   gtk_box_pack_start (GTK_BOX (box2), configbox, FALSE, TRUE, 0);
   gtk_widget_show (configbox);

   label = gtk_label_new("Device ");
   gtk_box_pack_start (GTK_BOX (configbox), label, FALSE, TRUE, 0);
   gtk_widget_show (label);

   device = gtk_label_new(device_str);
   gtk_box_pack_start (GTK_BOX (configbox), device, TRUE, TRUE, 0);
   gtk_misc_set_alignment (GTK_MISC (device), 0, 0.5);
   gtk_widget_show (device);

   label = gtk_label_new("Ratio ");
   gtk_box_pack_start (GTK_BOX (configbox), label, FALSE, TRUE, 0);
   gtk_widget_show (label);

   ratio = gtk_label_new(ratio_str);
   gtk_box_pack_start (GTK_BOX (configbox), ratio, TRUE, TRUE, 0);
   gtk_misc_set_alignment (GTK_MISC (ratio), 0, 0.5);
   gtk_widget_show (ratio);

   dirbox = gtk_hbox_new (FALSE, 0);
   gtk_container_border_width (GTK_CONTAINER (dirbox), 0);
   gtk_box_pack_start (GTK_BOX (box2), dirbox, FALSE, TRUE, 0);
   gtk_widget_show (dirbox);

   label = gtk_label_new("Directory: ");
   gtk_box_pack_start (GTK_BOX (dirbox), label, FALSE, TRUE, 0);
   gtk_widget_show (label);

   curdir = gtk_label_new(dir_str);
   gtk_box_pack_start (GTK_BOX (dirbox), curdir, TRUE, TRUE, 0);
   gtk_misc_set_alignment (GTK_MISC (curdir), 0, 0.5);
   gtk_widget_show (curdir);

#ifndef GTK_HAVE_FEATURES_1_1_0
   box3 = gtk_vbox_new (FALSE, 10);
   gtk_container_border_width (GTK_CONTAINER (box3), 10);
   gtk_box_pack_start (GTK_BOX (main_vbox), box3, TRUE, TRUE, 0);
   gtk_widget_show (box3);
#else
   scrolled_win = gtk_scrolled_window_new (NULL, NULL);
   gtk_container_set_border_width (GTK_CONTAINER (scrolled_win), 10);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_win),
				   GTK_POLICY_AUTOMATIC,
				   GTK_POLICY_AUTOMATIC);
#endif

   list = gtk_clist_new_with_titles (5, titles);
   gtk_clist_set_row_height (GTK_CLIST (list), 20);
#ifdef GTK_HAVE_FEATURES_1_1_0
   gtk_clist_set_button_actions(GTK_CLIST (list), 2, GTK_BUTTON_SELECTS);
#endif
   gtk_signal_connect (GTK_OBJECT (list), 
		       "select_row",
		       (GtkSignalFunc) select_list, 
		       NULL);

   gtk_clist_set_column_width (GTK_CLIST (list), 0, 40);
   gtk_clist_set_column_width (GTK_CLIST (list), 1, 120);
   gtk_clist_set_column_width (GTK_CLIST (list), 2, 60);
   gtk_clist_set_column_width (GTK_CLIST (list), 3, 90);
   gtk_clist_set_column_width (GTK_CLIST (list), 4, 60);

#ifndef GTK_HAVE_FEATURES_1_1_0
   gtk_clist_set_selection_mode (GTK_CLIST (list), GTK_SELECTION_SINGLE);
   gtk_clist_set_policy (GTK_CLIST (list),
			 GTK_POLICY_AUTOMATIC,
			 GTK_POLICY_AUTOMATIC);
#endif

   gtk_container_border_width (GTK_CONTAINER (list), 0);
#ifndef GTK_HAVE_FEATURES_1_1_0
   gtk_box_pack_start (GTK_BOX (box3), list, TRUE, TRUE, 0);
   gtk_widget_show (list);
#else
   gtk_container_add (GTK_CONTAINER (scrolled_win), list);
   gtk_widget_show (list);
   gtk_box_pack_start (GTK_BOX (main_vbox), scrolled_win, TRUE, TRUE, 0);
   gtk_widget_show (scrolled_win);
#endif

   hbox = gtk_hbutton_box_new ();
   gtk_container_border_width (GTK_CONTAINER (hbox), 10);
   gtk_box_pack_start (GTK_BOX (main_vbox), hbox, FALSE, TRUE, 0);
   gtk_widget_show (hbox);

   listbutton = gtk_button_new_with_label ("Acquire List");
   gtk_signal_connect_object (GTK_OBJECT (listbutton), "clicked",
			      GTK_SIGNAL_FUNC(acq_list),
			      GTK_OBJECT (window));
   gtk_container_add (GTK_CONTAINER (hbox), listbutton);
   GTK_WIDGET_SET_FLAGS (listbutton, GTK_CAN_DEFAULT);
   gtk_widget_grab_default (listbutton);
   gtk_widget_show (listbutton);

   button = gtk_button_new_with_label ("Working Dir");
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			      GTK_SIGNAL_FUNC(change_dir),
			      GTK_OBJECT (window));
   gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
   gtk_widget_show (button);

   savebutton = gtk_button_new_with_label ("Save All");
   gtk_signal_connect_object (GTK_OBJECT (savebutton), "clicked",
			      GTK_SIGNAL_FUNC(save_all),
			      GTK_OBJECT (window));
   gtk_container_add (GTK_CONTAINER (hbox), savebutton);
   gtk_widget_set_sensitive(GTK_WIDGET(savebutton), FALSE);
   gtk_widget_show (savebutton);
}

void
redraw_list ()
{
   char *texts[5];
   gint row, num = 0;
   struct pinfo_lst *p = proot;

   gtk_clist_freeze (GTK_CLIST (list));

   gtk_clist_clear (GTK_CLIST (list));
   while (num < status.numPictures && p != NULL)
     {
	texts[0] = p->number;
	texts[1] = p->filename;
	texts[2] = p->filesize;
	texts[3] = p->resolution;
	texts[4] = p->compression;
	row = gtk_clist_append (GTK_CLIST (list), texts);
	gtk_clist_set_text (GTK_CLIST (list), row, 0, p->number);
	gtk_clist_set_text (GTK_CLIST (list), row, 1, p->filename);
	gtk_clist_set_text (GTK_CLIST (list), row, 2, p->filesize);
	gtk_clist_set_text (GTK_CLIST (list), row, 3, p->resolution);
	gtk_clist_set_text (GTK_CLIST (list), row, 4, p->compression);
	num++;
	list_rows++;
	p = p->next;
     }

   gtk_clist_thaw (GTK_CLIST (list));
   gtk_widget_show (list);
}

static void
acq_list_cancel (void)
{
  progress_canceled = 1;
}

void acq_list(GtkWidget *widget, gpointer data)
{
   int i, ret_val;
   struct pinfo_lst *plst, *p;

   progress_canceled = 0;

   if (initialized == 0)
     {
	ret_val = init_dc210();
	if (ret_val == -1)
	  return;
	ret_val = load_status();
	if (ret_val == -1)
	  return;
	initialized = 1;
     }

   menus_enable_disable(FALSE);

   if (progress_box)
     progress_free (progress_box);
   progress_box = progress_new ("Getting List", "Acquiring image list...",
				(GtkSignalFunc) acq_list_cancel, 0);

   p = proot;
   while ( p != NULL)
     {
	plst = p->next;
	safe_free((void **)&p);
	p = plst;
     }
   proot = NULL;
	
   for ( i = 1 ; i <= (int)status.numPictures ; i++)
     {
	plst = safe_malloc(sizeof(struct pinfo_lst));
	if (proot == NULL)
	  {
	     proot = plst;
	     plst->prev = NULL;
	     plst->next = NULL;
	     p = plst;
	  }
	else
	  {
	     p->next = plst;
	     plst->next = NULL;
	     plst->prev = p;
	     p = plst;
	  }
	get_picture_info(i, plst);
	progress_update (progress_box, 
			 i / (gfloat) status.numPictures);

	while (gtk_events_pending ())
	  gtk_main_iteration ();

	if (progress_canceled == 1)
	  break;
     }
   progress_free (progress_box);
   progress_box = NULL;
   redraw_list();

   menus_enable_disable(TRUE);
}

void change_dir(GtkWidget *widget, gpointer data)
{
   file_selection(1);
}

void save_all(GtkWidget *widget, gpointer data)
{
   int i, ret_val;
   struct pinfo_lst *plst;
   unsigned char *temp;

   menus_enable_disable(FALSE);

   plst = proot;
   temp = safe_malloc(SHORT_STRING);
   for ( i = 1 ; i <= (int)status.numPictures && plst != NULL ; i++)
     {
	strcpy(temp, dir_str);
	strcat(temp, plst->filename);
	ret_val = get_picture(i, plst, temp);
	if (ret_val == -1)
	  {
	     safe_free((void **)&temp);
	     initialized = 0;
	     return;
	  } 
	plst = plst->next;
     }

   safe_free((void **)&temp);

   menus_enable_disable(TRUE);
}
