/*
 * Copyright (C) 1999 Toru Hoshina
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <locale.h>
#include <unistd.h>
#include <errno.h>
#include <pwd.h>
#include <sys/stat.h>
#include <sys/utsname.h>

#include <gtk/gtk.h>

#include "main.h"
#include "dialog.h"
#include "menu.h"
#include "list.h"
#include "lib.h"
#include "dc210.h"

GtkWidget *window = NULL;
GtkWidget *main_vbox = NULL;
GtkWidget *device = NULL;
GtkWidget *ratio = NULL;
GtkWidget *curdir = NULL;
GtkWidget *list = NULL;
GtkWidget *context_menu = NULL;
GtkWidget *listbutton = NULL;
GtkWidget *savebutton = NULL;
GtkWidget *fileSel = NULL;

char *rcpath = NULL;
gint list_rows = 0;
gint list_selected_row = 0;
gint device_no = 0;
gint ratio_no = 9600;
char *device_str = NULL;
char *ratio_str = NULL;
char *dir_str = NULL;
struct pinfo_lst *proot = NULL;
int initialized = 0;

void do_exit (int code)
{
   if (fileSel != NULL)
     gtk_widget_destroy (fileSel);

   gtk_widget_destroy (window);
   gtk_main_quit ();
}

int main(int argc, char *argv[])
{
  struct passwd *pw;
  char *p, *path = NULL, *temp = NULL;
  FILE *f;
  int ret_val;
  struct pinfo_lst *pl1, *pl2;

  gtk_set_locale();
  gtk_init(&argc, &argv);

  if ((p = getenv ("HOME")))
    path = safe_strdup(p);

  if (!path)
    if ((pw = getpwuid (getuid ())))
      path = safe_strdup(pw->pw_dir);

  rcpath = safe_malloc(strlen(path) + strlen("/.gtkrc") + 1);
  strcpy(rcpath, path);
  strcat(rcpath, "/.gtkrc");
  gtk_rc_parse(rcpath);
  safe_free((void **)&rcpath);

  rcpath = safe_malloc(strlen(path) + strlen("/.gkdcpirc") + 1);
  strcpy(rcpath, path);
  strcat(rcpath, "/.gkdcpirc");
  safe_free((void **)&path);

  temp = safe_malloc(SHORT_STRING);
  device_str = safe_malloc(SHORT_STRING);
  ratio_str = safe_malloc(SHORT_STRING);
  dir_str = safe_malloc(SHORT_STRING);
  device_str[0] = ratio_str[0] = dir_str[0] = '\0';

  if ((f = fopen (rcpath, "r")) != 0)
    {
      while( fgets (temp, SHORT_STRING, f) != NULL)
	{
	  p = strchr(temp, ':');
	  if (!p)
	    continue ;
	  else
	    {
	      *p++ = '\0';
	      *(p+strlen(p)-1) = '\0';
	      if (!strcmp(temp, "device"))
		{
		  strcpy(device_str, p);
		  device_no = *(device_str +(strlen(device_str)-1)) - '0';
		}
	      else if (!strcmp(temp, "ratio"))
		{
		  strcpy(ratio_str, p);
		  ratio_no = atoi(ratio_str);
		}
	      else if (!strcmp(temp, "dir"))
		{
		  strcpy(dir_str, p);
		}
	    }
	}
      fclose(f);
    }
  safe_free((void **)&temp);

  if (dir_str[0] == '\0')
    {
       strcpy(dir_str, rcpath);
       p = strrchr(dir_str, '/');
       *(p+1) = '\0';
    }

  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_signal_connect (GTK_OBJECT (window), "destroy",
		      GTK_SIGNAL_FUNC(do_exit), window);
  gtk_window_set_title(GTK_WINDOW(window), "Kodak Digital Camera Protocol Interface on GTK");
  gtk_widget_set_usize(GTK_WIDGET(window), 460, 500);
  gtk_window_set_policy(GTK_WINDOW(window), TRUE, TRUE, FALSE);
    
  main_vbox = gtk_vbox_new(FALSE, 0);
  gtk_container_border_width(GTK_CONTAINER(main_vbox), 1);
  gtk_container_add(GTK_CONTAINER(window), main_vbox);
  gtk_widget_show(main_vbox);
    
  init_menu();
  init_list();

  gtk_widget_show(window);

  if ((device_str[0] == '\0') || (ratio_str[0] == '\0') )
     {
	ret_val = alert_dialog("Please set preference first", "Yes", NULL, NULL);
	menu_sel_device(window, NULL);
	menu_sel_ratio(window, NULL);
	menu_save_rc(window, NULL);
     }

  gtk_main();

  if (initialized == 1)
     destroy_dc210();

  pl2 = proot;
  while ( pl2 != NULL)
    {
       pl1 = pl2->next;
       safe_free((void **)&pl2);
       pl2 = pl1;
    }

  safe_free((void **)&dir_str);
  safe_free((void **)&ratio_str);
  safe_free((void **)&device_str);
  safe_free((void **)&rcpath);

  exit(0);
}
